
/**
 Rendering Constants
 **/
static int YEAR_GAP = 1000;
static int COUNT_FILTER = 100;
static int MAP_COUNT_RADIUS = 48;
static int MAP_X = 600;
static int MAP_Y = 500;
static int MAP_Z = 1000;
color MIN_COLOR = color(204, 102, 0);
color MAX_COLOR = color(0, 102, 153);


void renderWords(Corpus corpus, int renderMode) {
  for (int i = 0; i < corpus.activePoints.size(); i++) {
    pushMatrix();
    translate(YEAR_GAP * i, 0, 0);
    for (Word w : corpus.activePoints.get(i)) {
      pushMatrix();
      w.render(corpus.selected, renderMode);
      popMatrix();
    }
    popMatrix();
  }
}

void renderCurves(Corpus corpus) {
  // Find locations of active coordinates for each selected word
  for (String sw : corpus.selected) {

    ArrayList<Coordinate> selectedCoordinates = new ArrayList<Coordinate>();
    for (int i = 0; i < corpus.activePoints.size(); i++) {
      if (corpus.activeQuickLookup.get(i).containsKey(sw)) {
        Integer wpos = corpus.activeQuickLookup.get(i).get(sw);
        selectedCoordinates.add(new Coordinate(corpus.activePoints.get(i).get(wpos).x + YEAR_GAP * i, 
          corpus.activePoints.get(i).get(wpos).y, 
          corpus.activePoints.get(i).get(wpos).z, 
          corpus.activePoints.get(i).get(wpos).c));
      }
    }

    // Draw connecting curves
    if (selectedCoordinates.size() >= 2) {
      noFill();
      strokeWeight(3);
      beginShape();
      curveVertex(selectedCoordinates.get(0).x, selectedCoordinates.get(0).y, selectedCoordinates.get(0).z);
      curveVertex(selectedCoordinates.get(0).x, selectedCoordinates.get(0).y, selectedCoordinates.get(0).z);

      for (int i = 1; i < selectedCoordinates.size() - 1; i++) {
        color c = selectedCoordinates.get(i).c;
        int r = (c >> 16) & 0xFF;
        int g = (c >> 8) & 0xFF;
        int b = c & 0xFF;
        r = constrain(r * 2, 0, 255);
        g = constrain(g * 2, 0, 255);
        b = constrain(b * 2, 0, 255);
        stroke(color(r, g, b), 255);

        // Add interpolation point if gap is greater than year gap
        if (selectedCoordinates.get(i).x - selectedCoordinates.get(i-1).x > YEAR_GAP * 1.5) {
          float midpoint = (selectedCoordinates.get(i).x + selectedCoordinates.get(i-1).x)/2.0;
          float ry = MAP_Y + 50;
          float rz = MAP_Z + 50;
          curveVertex(midpoint, ry, rz);
        }
        curveVertex(selectedCoordinates.get(i).x, selectedCoordinates.get(i).y, selectedCoordinates.get(i).z);
      }

      // Add interpolation for final point
      color c = selectedCoordinates.get(selectedCoordinates.size() - 1).c;
      int r = (c >> 16) & 0xFF;
      int g = (c >> 8) & 0xFF;
      int b = c & 0xFF;
      r = constrain(r * 2, 0, 255);
      g = constrain(g * 2, 0, 255);
      b = constrain(b * 2, 0, 255);
      stroke(color(r, g, b), 255);
      if (selectedCoordinates.get(selectedCoordinates.size() - 1).x - selectedCoordinates.get(selectedCoordinates.size() - 2).x > YEAR_GAP * 1.5) {
        float midpoint = (selectedCoordinates.get(selectedCoordinates.size() - 1).x + selectedCoordinates.get(selectedCoordinates.size() - 2).x)/2.0;
        float ry = MAP_Y * 2;
        float rz = MAP_Z * 2;
        curveVertex(midpoint, ry, rz);
      }
      curveVertex(selectedCoordinates.get(selectedCoordinates.size() - 1).x, selectedCoordinates.get(selectedCoordinates.size() - 1).y, selectedCoordinates.get(selectedCoordinates.size() - 1).z);
      curveVertex(selectedCoordinates.get(selectedCoordinates.size() - 1).x, selectedCoordinates.get(selectedCoordinates.size() - 1).y, selectedCoordinates.get(selectedCoordinates.size() - 1).z);
      endShape();
    }
  }
}

void renderArrowOfTime(Corpus corpus) {
  noFill();
  strokeWeight(2);
  stroke(255);
  beginShape();
  curveVertex(0, 0, 0);
  curveVertex(0, 0, 0);
  for (int i = 1; i < corpus.activePoints.size() - 1; i++) {
    curveVertex(YEAR_GAP * i, 0, 0);
  }  
  curveVertex(YEAR_GAP * (corpus.activePoints.size() - 1), 0, 0);
  curveVertex(YEAR_GAP * (corpus.activePoints.size() - 1), 0, 0);
  endShape();
}

void renderLabels(Corpus corpus) {
  // Draw Year Labels
  for (int i = 0; i < corpus.activePoints.size(); i++) {
    pushMatrix();
    noStroke();
    translate(YEAR_GAP * i, -MAP_Y * 1.5, 0);
    fill(255, 100);
    textAlign(CENTER, CENTER);
    textSize(64);
    textMode(MODEL);
    rotateY(-HALF_PI);
    text(Integer.toString(corpus.startYear + i), 0, 0);
    popMatrix();

    pushMatrix();
    translate(YEAR_GAP * i, 0, 0);
    fill(255, 50);
    sphere(20);
    popMatrix();
  }
}

/**
 Helper Functions
 **/
