import peasy.*;
import controlP5.*;
import java.util.*;

PeasyCam cam;
ControlP5 cp5;
Corpus corpus;
CheckBox trackingBox;
Textfield tf;


int currentYearFocus = 0;
int maxYear = 0;

boolean tracking = false;
String trackedWord = "";

boolean hideGUI = false;
boolean camReset = false;
int renderMode = 0;

int currentStart = 0;
int currentEnd = 100;

void setupData() {
  corpus = new Corpus("word_embeddings_year.csv");
}

void setupCamera() {
  cam = new PeasyCam(this, 100);
  float cameraZ = (height/2.0) / tan(PI*60.0/360.0);
  perspective(PI, width/height, 0.001, cameraZ*10.0);
}

void setupGUI() {
  cp5 = new ControlP5(this);
  PFont font = createFont("arial", 14);
  int current_y = 100;
  int y_spacing = 75;
  int text_y_spacing = 20;

  tf = cp5.addTextfield("selectedWordsController")
    .setPosition(50, current_y)
    .setSize(200, 40)
    .setFont(font)
    .setFocus(false)
    .setColor(color(255, 255, 255))
    .setAutoClear(false)
    .setCaptionLabel("Selected Words (seperate with commas)");

  tf.getCaptionLabel().setSize(14);

  trackingBox = cp5.addCheckBox("trackController")
    .setPosition(300, 100)
    .setSize(40, 40)
    .setFont(font)
    .setCaptionLabel("Track First")
    .addItem("Track?", 0);

  current_y += y_spacing;


  cp5.addRange("rangeController")
    .setBroadcast(false) 
    .setPosition(50, current_y)
    .setSize(400, 40)
    .setFont(font)
    .setHandleSize(20)
    .setRange(0, 1000)
    .setRangeValues(0, 100)
    // after the initialization we turn broadcast back on again
    .setBroadcast(true)
    .setColorForeground(color(255, 40))
    .setColorBackground(color(255, 40))  
    .setCaptionLabel("Select Top N Window")
    .getCaptionLabel().setSize(14).align(ControlP5.LEFT, ControlP5.BOTTOM_OUTSIDE);
  ;

  current_y += y_spacing;

  cp5.addButton("updateController")
    .setPosition(50, current_y)
    .setSize(100, 40)
    .setCaptionLabel("Submit")
    .getCaptionLabel().setSize(14);

  current_y += y_spacing;

  cp5.addTextlabel("controlLabel")
    .setText("Keyboard Controls")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 20));

  current_y += text_y_spacing * 2;
  cp5.addTextlabel("left")
    .setText("Left : Go to previous year")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
  current_y += text_y_spacing;
  cp5.addTextlabel("right")
    .setText("Right: Go to next year")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
  current_y += text_y_spacing;
  cp5.addTextlabel("h")
    .setText("h : Hide GUI")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
  current_y += text_y_spacing;
  cp5.addTextlabel("c")
    .setText("c : Clear selection")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
  current_y += text_y_spacing;
  cp5.addTextlabel("r")
    .setText("r : Reset camera")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
  current_y += text_y_spacing;
  cp5.addTextlabel("e")
    .setText("e : Submit")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
  current_y += text_y_spacing;
  cp5.addTextlabel("s")
    .setText("s : Sphere Render Mode")
    .setPosition(50, current_y)
    .setFont(createFont("Georgia", 14));
}

void controlEvent(ControlEvent theControlEvent) {
  if (theControlEvent.isFrom("rangeController")) {
    this.currentStart = (int) theControlEvent.getController().getArrayValue(0);
    this.currentEnd = (int) theControlEvent.getController().getArrayValue(1);
  } else if (theControlEvent.isFrom("updateController")) {
    updateEvent();
  } else if (theControlEvent.isFrom("trackController")) {
    if ((int) trackingBox.getArrayValue(0) == 1) {
      tracking = true;
    } else {
      tracking = false;
    }
  }
}

void updateEvent() {
  corpus.setActivePoints(this.currentStart, this.currentEnd);
  corpus.setSelectedWords(cp5.get(Textfield.class, "selectedWordsController").getText());
  trackedWord = corpus.selected.stream().findFirst().orElse("");
}

void keyPressed() {

  if (key == CODED && keyCode == RIGHT) {
    boolean moved = false;
    while (!moved) {
      if (currentYearFocus >= corpus.totalNYears) {
        moved = true;
      } else {
        currentYearFocus += 1;
        if (!tracking || trackedWord.equals("")) {
          cam.lookAt(YEAR_GAP * currentYearFocus, 0, 0);
          moved = true;
        } else {
          if (corpus.activeQuickLookup.get(currentYearFocus).containsKey(trackedWord)) {
            int idx = corpus.activeQuickLookup.get(currentYearFocus).get(trackedWord);
            Word w = corpus.activePoints.get(currentYearFocus).get(idx);
            cam.lookAt(YEAR_GAP * currentYearFocus + w.x, w.y, w.z);
            moved = true;
          }
        }
      }
    }
  } else if (key == CODED && keyCode == LEFT) {
    boolean moved = false;
    while (!moved) {
      if (currentYearFocus <= 0) {
        moved = true;
      } else {
        currentYearFocus -= 1;
        if (!tracking || trackedWord.equals("")) {
          cam.lookAt(YEAR_GAP * currentYearFocus, 0, 0);
          moved = true;
        } else {
          if (corpus.activeQuickLookup.get(currentYearFocus).containsKey(trackedWord)) {
            int idx = corpus.activeQuickLookup.get(currentYearFocus).get(trackedWord);
            Word w = corpus.activePoints.get(currentYearFocus).get(idx);
            cam.lookAt(YEAR_GAP * currentYearFocus + w.x, w.y, w.z);
            moved = true;
          }
        }
      }
    }
  } else if (!tf.isFocus() && key == 'h') {
    hideGUI = !hideGUI;
  } else if (!tf.isFocus() && key == 'c') {
    tf.setText("");
    updateEvent();
  } else if (!tf.isFocus() && key == 'r') {
    cam.reset();    
    cam.setDistance(1750);
  } else if (!tf.isFocus() && key == 'e') {
    updateEvent();
  } else if (!tf.isFocus() && key == 's') {
    renderMode = 1 - renderMode;
  } else if (!tf.isFocus() && key == 'p'){
    saveFrame();
    print("Frame saved!");
  }
}

void setup() {
  size(1920, 1080, P3D);
  setupData();
  setupCamera();
  setupGUI();

  cam.rotateY(-HALF_PI * .75);
  cam.rotateX(HALF_PI * .25);
  cam.setDistance(1750);

  cp5.setAutoDraw(false);
}

void gui() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}

void draw() {
  background(0);
  renderWords(corpus, renderMode);
  renderCurves(corpus);
  renderArrowOfTime(corpus);
  renderLabels(corpus);
  if (!hideGUI) {
    gui();
  }
}
